; ***********************************************************************************
; * Name		:	G. Mayer														*
; *	Description	:	Subroutines zur Dateneingabe ber Tasten bzw. Funk				*
; * Datum		:	10.03.2013 auf Mega16											*
; *																					*
; *  2013 dazkeiler.de																*
; *																					*
; *	Folgende Subroutines sind vorhanden												*
; * ===================================												*
; *																					*
; * DIPManagement																	*
; *	KeyManagement																	*
; *	Light01Management - Light08Management											*
; * UpdatePWMOUT01 - UpdatePWMOUT08													*
; *																					*
; ***********************************************************************************

; ***********************************************************************************
; * Tabellen																		*
; ***********************************************************************************
TAB_PWMValue:					
.db	40,70,100,130,160,190,220,255,0,0

; ***********************************************************************************
; * DIP Switch Management fr Eingang 8 und Ausgang 8								*
; *																					*
; * DIP01 | DIP02 |																	*
; * --------------------------------------------------------------------------------*
; *	  0   |   0	  |	Tastfunktion, keine PWM (kurzer Tastendruck)					*
; *	  0   |   1	  |	Tastfunktion, keine PWM	(kurzer und langer Tastendruck)			*
; *	  1   |   0	  |	Schaltfunktion, keine PWM (kurzer Tastendruck)					*					
; *	  1   |   1	  |	Normalbetrieb Ein/Aus/Dimmen/PWM (kurzer und langer Tastendruck)*
; ***********************************************************************************
DIPManagement:
	in		XL,DIPPort
	andi	XL,0b00000011

	cpi		XL,0
	brne	DIPManagement_NotCode0

	cbr		R_CONTROL,(1<<F_DIPNormalMode)
	lds		XL,RAM_KeyCode
	ldi		XH,K_Key08
	cpse	XL,XH
	rjmp	DIPManagement_End

	sbrs	R_CONTROL,F_TasteKurz						; wurde Taster kurz gedrckt dann nur Ein bzw. Ausschalten
	rjmp	DIPManagement_Code0NotPress	

DIPManagement_Code0Press:
	cbr		R_CONTROL,(1<<F_TasteKurz)
	cbr		R_CONTROL,(1<<F_TasteLang)
	sbi		LightPort,Light08
	ldi		XL,Low(K_TimeKeyOffShort)
	ldi		XH,High(K_TimeKeyOffShort)
	sts		RAM_TimeCounterL,XL				
	sts		RAM_TimeCounterH,XH	
	cbr		R_STATUS,(1<<F_TimeOut)	
	rjmp	DIPManagement_End

DIPManagement_Code0NotPress:
	sbrc	R_STATUS,F_TimeOut
	cbi		LightPort,Light08
	rjmp	DIPManagement_End

DIPManagement_NotCode0:
	cpi		XL,1
	brne	DIPManagement_NotCode1

	cbr		R_CONTROL,(1<<F_DIPNormalMode)
	lds		XL,RAM_KeyCode
	ldi		XH,K_Key08
	cpse	XL,XH
	rjmp	DIPManagement_End

	sbrc	R_CONTROL,F_TasteKurz						; wurde Taster kurz gedrckt dann nur Ein bzw. Ausschalten
	rjmp	DIPManagement_Code1_Press
	sbrs	R_CONTROL,F_TasteLang
	rjmp	DIPManagement_Code1_NotPress	

DIPManagement_Code1_Press:
	cbr		R_CONTROL,(1<<F_TasteKurz)
	cbr		R_CONTROL,(1<<F_TasteLang)
	sbi		LightPort,Light08
	ldi		XL,Low(K_TimeKeyOff)
	ldi		XH,High(K_TimeKeyOff)
	sts		RAM_TimeCounterL,XL				
	sts		RAM_TimeCounterH,XH	
	cbr		R_STATUS,(1<<F_TimeOut)	
	rjmp	DIPManagement_End

DIPManagement_Code1_NotPress:
	sbrc	R_STATUS,F_TimeOut
	cbi		LightPort,Light08
	rjmp	DIPManagement_End

DIPManagement_NotCode1:
	cpi		XL,2
	brne	DIPManagement_NotCode2

	cbr		R_CONTROL,(1<<F_DIPNormalMode)
	lds		XL,RAM_KeyCode
	ldi		XH,K_Key08
	cpse	XL,XH
	rjmp	DIPManagement_End

	sbrs	R_CONTROL,F_TasteKurz						; wurde Taster kurz gedrckt dann nur Ein bzw. Ausschalten
	rjmp	DIPManagement_End									

	cbr		R_CONTROL,(1<<F_TasteKurz)
	cbr		R_CONTROL,(1<<F_TasteLang)
	rcall	ToggleLight08	
	rjmp	DIPManagement_End

DIPManagement_NotCode2:
	cpi		XL,3
	brne	DIPManagement_NotCode3

	sbrc	R_CONTROL,F_DIPNormalMode
	rjmp	DIPManagement_End

	ldi		XL,K_PWMValueOff							; Lampe ausschalten
	sts		RAM_PWMSollValue8,XL
	ldi		XL,K_PWMLevelOff							; Counter auf MAX fr erneutes einschalten
	sts		RAM_PWMCounter8,XL
	sbr		R_CONTROL,(1<<F_DIPNormalMode)	
	rjmp	DIPManagement_End

DIPManagement_NotCode3:



DIPManagement_End:
	ret

; ***********************************************************************************
; * Tastenauswertung 																*
; ***********************************************************************************
KeyManagement:
;	in		R_LEDBuffer,KeyAndLEDPort					; Zustnde der LEDs merken

	; *******************************************************************************
	; * Port af Input PullUp umstellen												*
	; * PullUp 		: 0 -> Nein	, 1 -> Ja											*
	; * Richtung 	: 0 -> Input, 1 -> Output										*
	; *******************************************************************************
;	ldi		XL,0b11111111
;	ldi		XH,0b00000000
;	out		KeyAndLEDPort,XL
;	out		KeyAndLEDDDR,XH

	in		XL,KeyPort									; Keys einlesen

	; *******************************************************************************
	; * Taste akzeptieren (Mainboard)												*
	; *******************************************************************************
	sbrc	XL,Key01
	rjmp	KeyManagement_NotKey01

	ldi		R_KEYBuffer,K_Key01
	rjmp	KeyManagement_Key
	
KeyManagement_NotKey01:
	sbrc	XL,Key02
	rjmp	KeyManagement_NotKey02

	ldi		R_KEYBuffer,K_Key02
	rjmp	KeyManagement_Key

KeyManagement_NotKey02:
	sbrc	XL,Key03
	rjmp	KeyManagement_NotKey03

	ldi		R_KEYBuffer,K_Key03
	rjmp	KeyManagement_Key

KeyManagement_NotKey03:
	sbrc	XL,Key04
	rjmp	KeyManagement_NotKey04

	ldi		R_KEYBuffer,K_Key04
	rjmp	KeyManagement_Key

KeyManagement_NotKey04:
	sbrc	XL,Key05
	rjmp	KeyManagement_NotKey05

	ldi		R_KEYBuffer,K_Key05
	rjmp	KeyManagement_Key

KeyManagement_NotKey05:
	sbrc	XL,Key06
	rjmp	KeyManagement_NotKey06

	ldi		R_KEYBuffer,K_Key06
	rjmp	KeyManagement_Key

KeyManagement_NotKey06:
	sbrc	XL,Key07
	rjmp	KeyManagement_NotKey07

	ldi		R_KEYBuffer,K_Key07
	rjmp	KeyManagement_Key

KeyManagement_NotKey07:
	sbrc	XL,Key08
	rjmp	KeyManagement_NotKey08

	ldi		R_KEYBuffer,K_Key08
	rjmp	KeyManagement_Key

KeyManagement_NotKey08:
	rjmp	KeyManagement_NotKey

	; *******************************************************************************
	; * Taste																		*
	; *******************************************************************************
KeyManagement_Key:
	inc		R_TastenZeit
	cpi		R_TastenZeit,K_TimeKeyDim					; Prfen auf langen Tastendruck
	breq	KeyManagement_KeyOKFirstTime

	cpi		R_TastenZeit,K_TimeKeyDim+K_TimeKeyDimAuto
	brlo	KeyManagement_OffTimeReload	

	ldi		R_TastenZeit,K_TimeKeyDim					; Prellzeit neu initialisieren
	rjmp	KeyManagement_KeyOK

KeyManagement_KeyOKFirstTime:
	sbr		R_CONTROL,(1<<F_ChangeDirection)

KeyManagement_KeyOK:
	sbr		R_CONTROL,(1<<F_TasteLang)					; Dimmen freischalten
;	ldi		XL,255
;	sub		XL,R_KEYBuffer		
;	and		R_LEDBuffer,XL
	sts		RAM_KeyCode,R_KEYBuffer
	clr		R_KEYBuffer
	rjmp	KeyManagement_OffTimeReload

	; *******************************************************************************
	; * Tasten Auszeit neu laden													*
	; *******************************************************************************
KeyManagement_OffTimeReload:
	ldi		R_AuszeitTaste,K_AusZyclen
	rjmp	KeyManagement_End

KeyManagement_NotKey:
	; *******************************************************************************
	; * Keine Taste																	*
	; *******************************************************************************
	cpi		R_TastenZeit,K_TimeKeyDim
	brsh	KeyManagement_NotKeyNext

	cpi		R_TastenZeit,K_PrellZyclen
	brlo	KeyManagement_NotKeyNext

	sbr		R_CONTROL,(1<<F_TasteKurz)					; Taster wurde kurz gedrckt
;	eor		R_LEDBuffer,R_KEYBuffer
	sts		RAM_KeyCode,R_KEYBuffer
	clr		R_KEYBuffer
	clr		R_TastenZeit
		
KeyManagement_NotKeyNext:
	cpi		R_AuszeitTaste,0
	breq	KeyManagement_OffTimeZero

	dec		R_AuszeitTaste
	rjmp	KeyManagement_End

KeyManagement_OffTimeZero:								; Aus erkannt
	cbr		R_CONTROL,((1<<F_TasteKurz)|(1<<F_TasteLang))
	clr		R_TastenZeit

KeyManagement_End:
;	ldi		XH,0b11111111
;	out		KeyAndLEDDDR,XH								; Port auf Ausgang schalten
;	out		KeyAndLEDPort,R_LEDBuffer
	ret

; ***********************************************************************************
; * Tasten Management fr Taster 1													*
; ***********************************************************************************
Light01Management:
	lds		XL,RAM_KeyCode
	cpi		XL,K_Key01
	brne	Light01Management_End
	
	sbrc	R_CONTROL,F_TasteLang						; wird Taster lange gedrckt...
	rjmp	Light01Management_Dimmen					; ...dann zu Dimmen

	sbrs	R_CONTROL,F_TasteKurz						; wurde Taster kurz gedrckt dann nur Ein bzw. Ausschalten
	rjmp	Light01Management_End									

	cbr		R_CONTROL,(1<<F_TasteKurz)
	lds		XL,RAM_PWMIstValue1							; ist Ausgang EIN, dann Ausschalten
	cpi		XL,K_PWMValueOff
	breq	Light01Management_SetPWMValue				; ...sonst Output Wert setzen

Light01Management_OUTOff:
	ldi		XL,K_PWMValueOff							; Lampe ausschalten
	sts		RAM_PWMSollValue1,XL
	ldi		XL,K_PWMLevelMax							; Counter auf MAX fr erneutes einschalten
	sts		RAM_PWMCounter1,XL
	rjmp	Light01Management_EndWithTasteCodeReset

Light01Management_Dimmen:
	cbr		R_CONTROL,(1<<F_TasteLang)
	lds		XL,RAM_PWMCounter1	
	sbrc	R_CONTROL,F_ChangeDirection
	com		XL
;	sbrs	R_CONTROL,F_ChangeDirection
	inc		XL
	sts		RAM_PWMCounter1,XL
	cbr		R_CONTROL,(1<<F_ChangeDirection)

Light01Management_SetPWMValue:
	lds		XL,RAM_PWMCounter1
	sbrc	XL,3
	com		XL
	andi	XL,0b00000111								; berlaufen verriegeln, Counter luft hoch und wieder runter
	ldi		ZL,Low(TAB_PWMValue*2)						; Tabellen Anfang laden
	ldi		ZH,High(TAB_PWMValue*2)
	add		ZL,XL										; Counter zum Tabellen Anfang dazu addieren
	lpm		XL,Z										; PWM Wert aus Tabelle holen
	sts		RAM_PWMSollValue1,XL

Light01Management_EndWithTasteCodeReset:
	clr		XL
	sts		RAM_KeyCode,XL

Light01Management_End:
	ret

; ***********************************************************************************
; * Tasten Management fr Taster 2													*
; ***********************************************************************************
Light02Management:
	lds		XL,RAM_KeyCode
	cpi		XL,K_Key02
	brne	Light02Management_End
	
	sbrc	R_CONTROL,F_TasteLang						; wird Taster lange gedrckt...
	rjmp	Light02Management_Dimmen					; ...dann zu Dimmen

	sbrs	R_CONTROL,F_TasteKurz						; wurde Taster kurz gedrckt dann nur Ein bzw. Ausschalten
	rjmp	Light02Management_End									

	cbr		R_CONTROL,(1<<F_TasteKurz)
	lds		XL,RAM_PWMIstValue2							; ist Ausgang EIN, dann Ausschalten
	cpi		XL,K_PWMValueOff
	breq	Light02Management_SetPWMValue				; ...sonst Output Wert setzen

Light02Management_OUTOff:
	ldi		XL,K_PWMValueOff							; Lampe ausschalten
	sts		RAM_PWMSollValue2,XL
	ldi		XL,K_PWMLevelMax							; Counter auf MAX fr erneutes einschalten
	sts		RAM_PWMCounter2,XL
	rjmp	Light02Management_EndWithTasteCodeReset

Light02Management_Dimmen:
	cbr		R_CONTROL,(1<<F_TasteLang)
	lds		XL,RAM_PWMCounter2	
	sbrc	R_CONTROL,F_ChangeDirection
	com		XL
;	sbrs	R_CONTROL,F_ChangeDirection
	inc		XL
	sts		RAM_PWMCounter2,XL
	cbr		R_CONTROL,(1<<F_ChangeDirection)

Light02Management_SetPWMValue:
	lds		XL,RAM_PWMCounter2
	sbrc	XL,3
	com		XL
	andi	XL,0b00000111								; berlaufen verriegeln, Counter luft hoch und wieder runter
	ldi		ZL,Low(TAB_PWMValue*2)						; Tabellen Anfang laden
	ldi		ZH,High(TAB_PWMValue*2)
	add		ZL,XL										; Counter zum Tabellen Anfang dazu addieren
	lpm		XL,Z										; PWM Wert aus Tabelle holen
	sts		RAM_PWMSollValue2,XL

Light02Management_EndWithTasteCodeReset:
	clr		XL
	sts		RAM_KeyCode,XL

Light02Management_End:
	ret

; ***********************************************************************************
; * Tasten Management fr Taster 3													*
; ***********************************************************************************
Light03Management:
	lds		XL,RAM_KeyCode
	cpi		XL,K_Key03
	brne	Light03Management_End
	
	sbrc	R_CONTROL,F_TasteLang						; wird Taster lange gedrckt...
	rjmp	Light03Management_Dimmen					; ...dann zu Dimmen

	sbrs	R_CONTROL,F_TasteKurz						; wurde Taster kurz gedrckt dann nur Ein bzw. Ausschalten
	rjmp	Light03Management_End									

	cbr		R_CONTROL,(1<<F_TasteKurz)
	lds		XL,RAM_PWMIstValue3							; ist Ausgang EIN, dann Ausschalten
	cpi		XL,K_PWMValueOff
	breq	Light03Management_SetPWMValue				; ...sonst Output Wert setzen

Light03Management_OUTOff:
	ldi		XL,K_PWMValueOff							; Lampe ausschalten
	sts		RAM_PWMSollValue3,XL
	ldi		XL,K_PWMLevelMax							; Counter auf MAX fr erneutes einschalten
	sts		RAM_PWMCounter3,XL
	rjmp	Light03Management_EndWithTasteCodeReset

Light03Management_Dimmen:
	cbr		R_CONTROL,(1<<F_TasteLang)
	lds		XL,RAM_PWMCounter3	
	sbrc	R_CONTROL,F_ChangeDirection
	com		XL
;	sbrs	R_CONTROL,F_ChangeDirection
	inc		XL
	sts		RAM_PWMCounter3,XL
	cbr		R_CONTROL,(1<<F_ChangeDirection)

Light03Management_SetPWMValue:
	lds		XL,RAM_PWMCounter3
	sbrc	XL,3
	com		XL
	andi	XL,0b00000111								; berlaufen verriegeln, Counter luft hoch und wieder runter
	ldi		ZL,Low(TAB_PWMValue*2)						; Tabellen Anfang laden
	ldi		ZH,High(TAB_PWMValue*2)
	add		ZL,XL										; Counter zum Tabellen Anfang dazu addieren
	lpm		XL,Z										; PWM Wert aus Tabelle holen
	sts		RAM_PWMSollValue3,XL

Light03Management_EndWithTasteCodeReset:
	clr		XL
	sts		RAM_KeyCode,XL

Light03Management_End:
	ret

; ***********************************************************************************
; * Tasten Management fr Taster 4													*
; ***********************************************************************************
Light04Management:
	lds		XL,RAM_KeyCode
	cpi		XL,K_Key04
	brne	Light04Management_End
	
	sbrc	R_CONTROL,F_TasteLang						; wird Taster lange gedrckt...
	rjmp	Light04Management_Dimmen					; ...dann zu Dimmen

	sbrs	R_CONTROL,F_TasteKurz						; wurde Taster kurz gedrckt dann nur Ein bzw. Ausschalten
	rjmp	Light04Management_End									

	cbr		R_CONTROL,(1<<F_TasteKurz)
	lds		XL,RAM_PWMIstValue4							; ist Ausgang EIN, dann Ausschalten
	cpi		XL,K_PWMValueOff
	breq	Light04Management_SetPWMValue				; ...sonst Output Wert setzen

Light04Management_OUTOff:
	ldi		XL,K_PWMValueOff							; Lampe ausschalten
	sts		RAM_PWMSollValue4,XL
	ldi		XL,K_PWMLevelMax							; Counter auf MAX fr erneutes einschalten
	sts		RAM_PWMCounter4,XL
	rjmp	Light04Management_EndWithTasteCodeReset

Light04Management_Dimmen:
	cbr		R_CONTROL,(1<<F_TasteLang)
	lds		XL,RAM_PWMCounter4
	sbrc	R_CONTROL,F_ChangeDirection
	com		XL
;	sbrs	R_CONTROL,F_ChangeDirection
	inc		XL
	sts		RAM_PWMCounter4,XL
	cbr		R_CONTROL,(1<<F_ChangeDirection)

Light04Management_SetPWMValue:
	lds		XL,RAM_PWMCounter4
	sbrc	XL,3
	com		XL
	andi	XL,0b00000111								; berlaufen verriegeln, Counter luft hoch und wieder runter
	ldi		ZL,Low(TAB_PWMValue*2)						; Tabellen Anfang laden
	ldi		ZH,High(TAB_PWMValue*2)
	add		ZL,XL										; Counter zum Tabellen Anfang dazu addieren
	lpm		XL,Z										; PWM Wert aus Tabelle holen
	sts		RAM_PWMSollValue4,XL

Light04Management_EndWithTasteCodeReset:
	clr		XL
	sts		RAM_KeyCode,XL

Light04Management_End:
	ret

; ***********************************************************************************
; * Tasten Management fr Taster 5													*
; ***********************************************************************************
Light05Management:
	lds		XL,RAM_KeyCode
	cpi		XL,K_Key05
	brne	Light05Management_End
	
	sbrc	R_CONTROL,F_TasteLang						; wird Taster lange gedrckt...
	rjmp	Light05Management_Dimmen					; ...dann zu Dimmen

	sbrs	R_CONTROL,F_TasteKurz						; wurde Taster kurz gedrckt dann nur Ein bzw. Ausschalten
	rjmp	Light05Management_End									

	cbr		R_CONTROL,(1<<F_TasteKurz)
	lds		XL,RAM_PWMIstValue5							; ist Ausgang EIN, dann Ausschalten
	cpi		XL,K_PWMValueOff
	breq	Light05Management_SetPWMValue				; ...sonst Output Wert setzen

Light05Management_OUTOff:
	ldi		XL,K_PWMValueOff							; Lampe ausschalten
	sts		RAM_PWMSollValue5,XL
	ldi		XL,K_PWMLevelMax							; Counter auf MAX fr erneutes einschalten
	sts		RAM_PWMCounter5,XL
	rjmp	Light05Management_EndWithTasteCodeReset

Light05Management_Dimmen:
	cbr		R_CONTROL,(1<<F_TasteLang)
	lds		XL,RAM_PWMCounter5
	sbrc	R_CONTROL,F_ChangeDirection
	com		XL
;	sbrs	R_CONTROL,F_ChangeDirection
	inc		XL
	sts		RAM_PWMCounter5,XL
	cbr		R_CONTROL,(1<<F_ChangeDirection)

Light05Management_SetPWMValue:
	lds		XL,RAM_PWMCounter5
	sbrc	XL,3
	com		XL
	andi	XL,0b00000111								; berlaufen verriegeln, Counter luft hoch und wieder runter
	ldi		ZL,Low(TAB_PWMValue*2)						; Tabellen Anfang laden
	ldi		ZH,High(TAB_PWMValue*2)
	add		ZL,XL										; Counter zum Tabellen Anfang dazu addieren
	lpm		XL,Z										; PWM Wert aus Tabelle holen
	sts		RAM_PWMSollValue5,XL

Light05Management_EndWithTasteCodeReset:
	clr		XL
	sts		RAM_KeyCode,XL

Light05Management_End:
	ret

; ***********************************************************************************
; * Tasten Management fr Taster 6													*
; ***********************************************************************************
Light06Management:
	lds		XL,RAM_KeyCode
	cpi		XL,K_Key06
	brne	Light06Management_End
	
	sbrc	R_CONTROL,F_TasteLang						; wird Taster lange gedrckt...
	rjmp	Light06Management_Dimmen					; ...dann zu Dimmen

	sbrs	R_CONTROL,F_TasteKurz						; wurde Taster kurz gedrckt dann nur Ein bzw. Ausschalten
	rjmp	Light06Management_End									

	cbr		R_CONTROL,(1<<F_TasteKurz)
	lds		XL,RAM_PWMIstValue6							; ist Ausgang EIN, dann Ausschalten
	cpi		XL,K_PWMValueOff
	breq	Light06Management_SetPWMValue				; ...sonst Output Wert setzen

Light06Management_OUTOff:
	ldi		XL,K_PWMValueOff							; Lampe ausschalten
	sts		RAM_PWMSollValue6,XL
	ldi		XL,K_PWMLevelMax							; Counter auf MAX fr erneutes einschalten
	sts		RAM_PWMCounter6,XL
	rjmp	Light06Management_EndWithTasteCodeReset

Light06Management_Dimmen:
	cbr		R_CONTROL,(1<<F_TasteLang)
	lds		XL,RAM_PWMCounter6	
	sbrc	R_CONTROL,F_ChangeDirection
	com		XL
;	sbrs	R_CONTROL,F_ChangeDirection
	inc		XL
	sts		RAM_PWMCounter6,XL
	cbr		R_CONTROL,(1<<F_ChangeDirection)

Light06Management_SetPWMValue:
	lds		XL,RAM_PWMCounter6
	sbrc	XL,3
	com		XL
	andi	XL,0b00000111								; berlaufen verriegeln, Counter luft hoch und wieder runter
	ldi		ZL,Low(TAB_PWMValue*2)						; Tabellen Anfang laden
	ldi		ZH,High(TAB_PWMValue*2)
	add		ZL,XL										; Counter zum Tabellen Anfang dazu addieren
	lpm		XL,Z										; PWM Wert aus Tabelle holen
	sts		RAM_PWMSollValue6,XL

Light06Management_EndWithTasteCodeReset:
	clr		XL
	sts		RAM_KeyCode,XL

Light06Management_End:
	ret

; ***********************************************************************************
; * Tasten Management fr Taster 7													*
; ***********************************************************************************
Light07Management:
	lds		XL,RAM_KeyCode
	cpi		XL,K_Key07
	brne	Light07Management_End
	
	sbrc	R_CONTROL,F_TasteLang						; wird Taster lange gedrckt...
	rjmp	Light07Management_Dimmen					; ...dann zu Dimmen

	sbrs	R_CONTROL,F_TasteKurz						; wurde Taster kurz gedrckt dann nur Ein bzw. Ausschalten
	rjmp	Light07Management_End									

	cbr		R_CONTROL,(1<<F_TasteKurz)
	lds		XL,RAM_PWMIstValue7							; ist Ausgang EIN, dann Ausschalten
	cpi		XL,K_PWMValueOff
	breq	Light07Management_SetPWMValue				; ...sonst Output Wert setzen

Light07Management_OUTOff:
	ldi		XL,K_PWMValueOff							; Lampe ausschalten
	sts		RAM_PWMSollValue7,XL
	ldi		XL,K_PWMLevelMax							; Counter auf MAX fr erneutes einschalten
	sts		RAM_PWMCounter7,XL
	rjmp	Light07Management_EndWithTasteCodeReset

Light07Management_Dimmen:
	cbr		R_CONTROL,(1<<F_TasteLang)
	lds		XL,RAM_PWMCounter7	
	sbrc	R_CONTROL,F_ChangeDirection
	com		XL
;	sbrs	R_CONTROL,F_ChangeDirection
	inc		XL
	sts		RAM_PWMCounter7,XL
	cbr		R_CONTROL,(1<<F_ChangeDirection)

Light07Management_SetPWMValue:
	lds		XL,RAM_PWMCounter7
	sbrc	XL,3
	com		XL
	andi	XL,0b00000111								; berlaufen verriegeln, Counter luft hoch und wieder runter
	ldi		ZL,Low(TAB_PWMValue*2)						; Tabellen Anfang laden
	ldi		ZH,High(TAB_PWMValue*2)
	add		ZL,XL										; Counter zum Tabellen Anfang dazu addieren
	lpm		XL,Z										; PWM Wert aus Tabelle holen
	sts		RAM_PWMSollValue7,XL

Light07Management_EndWithTasteCodeReset:
	clr		XL
	sts		RAM_KeyCode,XL

Light07Management_End:
	ret

; ***********************************************************************************
; * Tasten Management fr Taster 8													*
; ***********************************************************************************
Light08Management:
	sbrs	R_CONTROL,F_DIPNormalMode
	rjmp	Light08Management_End

	lds		XL,RAM_KeyCode
	cpi		XL,K_Key08
	brne	Light08Management_End
	
	sbrc	R_CONTROL,F_TasteLang						; wird Taster lange gedrckt...
	rjmp	Light08Management_Dimmen					; ...dann zu Dimmen

	sbrs	R_CONTROL,F_TasteKurz						; wurde Taster kurz gedrckt dann nur Ein bzw. Ausschalten
	rjmp	Light08Management_End									

	cbr		R_CONTROL,(1<<F_TasteKurz)
	lds		XL,RAM_PWMIstValue8							; ist Ausgang EIN, dann Ausschalten
	cpi		XL,K_PWMValueOff
	breq	Light08Management_SetPWMValue				; ...sonst Output Wert setzen

Light08Management_OUTOff:
	ldi		XL,K_PWMValueOff							; Lampe ausschalten
	sts		RAM_PWMSollValue8,XL
	ldi		XL,K_PWMLevelMax							; Counter auf MAX fr erneutes einschalten
	sts		RAM_PWMCounter8,XL
	rjmp	Light08Management_EndWithTasteCodeReset

Light08Management_Dimmen:
	cbr		R_CONTROL,(1<<F_TasteLang)
	lds		XL,RAM_PWMCounter8	
	sbrc	R_CONTROL,F_ChangeDirection
	com		XL
;	sbrs	R_CONTROL,F_ChangeDirection
	inc		XL
	sts		RAM_PWMCounter8,XL
	cbr		R_CONTROL,(1<<F_ChangeDirection)

Light08Management_SetPWMValue:
	lds		XL,RAM_PWMCounter8
	sbrc	XL,3
	com		XL
	andi	XL,0b00000111								; berlaufen verriegeln, Counter luft hoch und wieder runter
	ldi		ZL,Low(TAB_PWMValue*2)						; Tabellen Anfang laden
	ldi		ZH,High(TAB_PWMValue*2)
	add		ZL,XL										; Counter zum Tabellen Anfang dazu addieren
	lpm		XL,Z										; PWM Wert aus Tabelle holen
	sts		RAM_PWMSollValue8,XL

Light08Management_EndWithTasteCodeReset:
	clr		XL
	sts		RAM_KeyCode,XL

Light08Management_End:
	ret

; ***********************************************************************************
; * PWM Ausgabe aktualisieren fr Ausgang 1											*
; ***********************************************************************************
UpdatePWMOUT01:
	lds		XL,RAM_PWMSollValue1
	cpi		XL,K_PWMValueOff;0									; ist PWM Wert > 0...
	breq	UpdatePWMOUT01_XLToPWMIstValue						

UpdatePWMOUT01_NotZero:											; Sollwert ist nicht 0, abfrage ob PWM bereits luft
	lds		XL,RAM_PWMIstValue1
	cpi		XL,K_PWMValueOff
	brne	UpdatePWMOUT01_Run									; ...sonst weiter mit RUN dec oder inc

	lds		XL,K_PWMValueStart									; Sanftanlauf Startwert initialisieren
	ldi		XH,K_PWMSchrittweiteLangsam
	sts		RAM_PWMSchrittweite1,XH								; Sanftanlauf langsam einstellen
;	rjmp	UpdatePWMOUT01_XLToPWMIstValue
	rjmp	UpdatePWMOUT01_Inc

UpdatePWMOUT01_Run:
	lds		XL,RAM_PWMIstValue1
	lds		XH,RAM_PWMSollValue1
	cp		XL,XH												; ist IST-Wert < SOLL-Wert...
	brlo	UpdatePWMOUT01_Inc									; ...dann PWM erhhen

	sts		RAM_PWMIstValue1,XH									; ...sonst SOLL-Wert in  IST-Wert laden
	ldi		XH,K_PWMSchrittweiteSchnell	
	sts		RAM_PWMSchrittweite1,XH								; Sanftanlauf auf schnell umschalten
	rjmp	UpdatePWMOUT01_End

UpdatePWMOUT01_Inc:
	lds		XL,RAM_PWMIstValue1
	lds		XH,RAM_PWMSchrittweite1
	add		XL,XH												; Schrittweite zum OCR0A addieren...
	brcs	UpdatePWMOUT01_Limit								; und prfen ob nchster Wert einen berlauf ergiebt, wenn "NEIN" dann nchste Abfrage sonst Limit erreicht...

	lds		XH,RAM_PWMSollValue1
	cp		XH,XL												; ...kein berlauf -> Abfrage ist SOLL-Wert >= IST-Wert, wenn "JA"...
	brsh	UpdatePWMOUT01_XLToPWMIstValue						; ...dann zu Ende 

UpdatePWMOUT01_Limit:
	lds		XL,RAM_PWMSollValue1								; ...NEIN bzw. berlauf SOLL-Wert in IST-Wet bernehmen

UpdatePWMOUT01_XLToPWMIstValue:
	sts		RAM_PWMIstValue1,XL

UpdatePWMOUT01_End:
	ret
	
; ***********************************************************************************
; * PWM Ausgabe aktualisieren fr Ausgang 2											*
; ***********************************************************************************
UpdatePWMOUT02:
	lds		XL,RAM_PWMSollValue2
	cpi		XL,K_PWMValueOff;0									; ist PWM Wert > 0...
	breq	UpdatePWMOUT02_XLToPWMIstValue						

UpdatePWMOUT02_NotZero:											; Sollwert ist nicht 0, abfrage ob PWM bereits luft
	lds		XL,RAM_PWMIstValue2
	cpi		XL,K_PWMValueOff
	brne	UpdatePWMOUT02_Run									; ...sonst weiter mit RUN dec oder inc

	lds		XL,K_PWMValueStart									; Sanftanlauf Startwert initialisieren
	ldi		XH,K_PWMSchrittweiteLangsam
	sts		RAM_PWMSchrittweite2,XH								; Sanftanlauf langsam einstellen
;	rjmp	UpdatePWMOUT02_XLToPWMIstValue
	rjmp	UpdatePWMOUT02_Inc

UpdatePWMOUT02_Run:
	lds		XL,RAM_PWMIstValue2
	lds		XH,RAM_PWMSollValue2
	cp		XL,XH												; ist IST-Wert < SOLL-Wert...
	brlo	UpdatePWMOUT02_Inc									; ...dann PWM erhhen

	sts		RAM_PWMIstValue2,XH									; ...sonst SOLL-Wert in  IST-Wert laden
	ldi		XH,K_PWMSchrittweiteSchnell	
	sts		RAM_PWMSchrittweite2,XH								; Sanftanlauf auf schnell umschalten
	rjmp	UpdatePWMOUT02_End

UpdatePWMOUT02_Inc:
	lds		XL,RAM_PWMIstValue2
	lds		XH,RAM_PWMSchrittweite2
	add		XL,XH												; Schrittweite zum OCR0A addieren...
	brcs	UpdatePWMOUT02_Limit								; und prfen ob nchster Wert einen berlauf ergiebt, wenn "NEIN" dann nchste Abfrage sonst Limit erreicht...

	lds		XH,RAM_PWMSollValue2
	cp		XH,XL												; ...kein berlauf -> Abfrage ist SOLL-Wert >= IST-Wert, wenn "JA"...
	brsh	UpdatePWMOUT02_XLToPWMIstValue						; ...dann zu Ende 

UpdatePWMOUT02_Limit:
	lds		XL,RAM_PWMSollValue2								; ...NEIN bzw. berlauf SOLL-Wert in IST-Wet bernehmen

UpdatePWMOUT02_XLToPWMIstValue:
	sts		RAM_PWMIstValue2,XL

UpdatePWMOUT02_End:
	ret

; ***********************************************************************************
; * PWM Ausgabe aktualisieren fr Ausgang 3											*
; ***********************************************************************************
UpdatePWMOUT03:
	lds		XL,RAM_PWMSollValue3
	cpi		XL,K_PWMValueOff;0									; ist PWM Wert > 0...
	breq	UpdatePWMOUT03_XLToPWMIstValue						

UpdatePWMOUT03_NotZero:											; Sollwert ist nicht 0, abfrage ob PWM bereits luft
	lds		XL,RAM_PWMIstValue3
	cpi		XL,K_PWMValueOff
	brne	UpdatePWMOUT03_Run									; ...sonst weiter mit RUN dec oder inc

	lds		XL,K_PWMValueStart									; Sanftanlauf Startwert initialisieren
	ldi		XH,K_PWMSchrittweiteLangsam
	sts		RAM_PWMSchrittweite3,XH								; Sanftanlauf langsam einstellen
;	rjmp	UpdatePWMOUT03_XLToPWMIstValue
	rjmp	UpdatePWMOUT03_Inc

UpdatePWMOUT03_Run:
	lds		XL,RAM_PWMIstValue3
	lds		XH,RAM_PWMSollValue3
	cp		XL,XH												; ist IST-Wert < SOLL-Wert...
	brlo	UpdatePWMOUT03_Inc									; ...dann PWM erhhen

	sts		RAM_PWMIstValue3,XH									; ...sonst SOLL-Wert in  IST-Wert laden
	ldi		XH,K_PWMSchrittweiteSchnell	
	sts		RAM_PWMSchrittweite3,XH								; Sanftanlauf auf schnell umschalten
	rjmp	UpdatePWMOUT03_End

UpdatePWMOUT03_Inc:
	lds		XL,RAM_PWMIstValue3
	lds		XH,RAM_PWMSchrittweite3
	add		XL,XH												; Schrittweite zum OCR0A addieren...
	brcs	UpdatePWMOUT03_Limit								; und prfen ob nchster Wert einen berlauf ergiebt, wenn "NEIN" dann nchste Abfrage sonst Limit erreicht...

	lds		XH,RAM_PWMSollValue3
	cp		XH,XL												; ...kein berlauf -> Abfrage ist SOLL-Wert >= IST-Wert, wenn "JA"...
	brsh	UpdatePWMOUT03_XLToPWMIstValue						; ...dann zu Ende 

UpdatePWMOUT03_Limit:
	lds		XL,RAM_PWMSollValue3								; ...NEIN bzw. berlauf SOLL-Wert in IST-Wet bernehmen

UpdatePWMOUT03_XLToPWMIstValue:
	sts		RAM_PWMIstValue3,XL

UpdatePWMOUT03_End:
	ret

; ***********************************************************************************
; * PWM Ausgabe aktualisieren fr Ausgang 4											*
; ***********************************************************************************
UpdatePWMOUT04:
	lds		XL,RAM_PWMSollValue4
	cpi		XL,K_PWMValueOff;0									; ist PWM Wert > 0...
	breq	UpdatePWMOUT04_XLToPWMIstValue						

UpdatePWMOUT04_NotZero:											; Sollwert ist nicht 0, abfrage ob PWM bereits luft
	lds		XL,RAM_PWMIstValue4
	cpi		XL,K_PWMValueOff
	brne	UpdatePWMOUT04_Run									; ...sonst weiter mit RUN dec oder inc

	lds		XL,K_PWMValueStart									; Sanftanlauf Startwert initialisieren
	ldi		XH,K_PWMSchrittweiteLangsam
	sts		RAM_PWMSchrittweite4,XH								; Sanftanlauf langsam einstellen
;	rjmp	UpdatePWMOUT04_XLToPWMIstValue
	rjmp	UpdatePWMOUT04_Inc

UpdatePWMOUT04_Run:
	lds		XL,RAM_PWMIstValue4
	lds		XH,RAM_PWMSollValue4
	cp		XL,XH												; ist IST-Wert < SOLL-Wert...
	brlo	UpdatePWMOUT04_Inc									; ...dann PWM erhhen

	sts		RAM_PWMIstValue4,XH									; ...sonst SOLL-Wert in  IST-Wert laden
	ldi		XH,K_PWMSchrittweiteSchnell	
	sts		RAM_PWMSchrittweite4,XH								; Sanftanlauf auf schnell umschalten
	rjmp	UpdatePWMOUT04_End

UpdatePWMOUT04_Inc:
	lds		XL,RAM_PWMIstValue4
	lds		XH,RAM_PWMSchrittweite4
	add		XL,XH												; Schrittweite zum OCR0A addieren...
	brcs	UpdatePWMOUT04_Limit								; und prfen ob nchster Wert einen berlauf ergiebt, wenn "NEIN" dann nchste Abfrage sonst Limit erreicht...

	lds		XH,RAM_PWMSollValue4
	cp		XH,XL												; ...kein berlauf -> Abfrage ist SOLL-Wert >= IST-Wert, wenn "JA"...
	brsh	UpdatePWMOUT04_XLToPWMIstValue						; ...dann zu Ende 

UpdatePWMOUT04_Limit:
	lds		XL,RAM_PWMSollValue4								; ...NEIN bzw. berlauf SOLL-Wert in IST-Wet bernehmen

UpdatePWMOUT04_XLToPWMIstValue:
	sts		RAM_PWMIstValue4,XL

UpdatePWMOUT04_End:
	ret

; ***********************************************************************************
; * PWM Ausgabe aktualisieren fr Ausgang 5											*
; ***********************************************************************************
UpdatePWMOUT05:
	lds		XL,RAM_PWMSollValue5
	cpi		XL,K_PWMValueOff;0									; ist PWM Wert > 0...
	breq	UpdatePWMOUT05_XLToPWMIstValue						

UpdatePWMOUT05_NotZero:											; Sollwert ist nicht 0, abfrage ob PWM bereits luft
	lds		XL,RAM_PWMIstValue5
	cpi		XL,K_PWMValueOff
	brne	UpdatePWMOUT05_Run									; ...sonst weiter mit RUN dec oder inc

	lds		XL,K_PWMValueStart									; Sanftanlauf Startwert initialisieren
	ldi		XH,K_PWMSchrittweiteLangsam
	sts		RAM_PWMSchrittweite5,XH								; Sanftanlauf langsam einstellen
;	rjmp	UpdatePWMOUT05_XLToPWMIstValue
	rjmp	UpdatePWMOUT05_Inc

UpdatePWMOUT05_Run:
	lds		XL,RAM_PWMIstValue5
	lds		XH,RAM_PWMSollValue5
	cp		XL,XH												; ist IST-Wert < SOLL-Wert...
	brlo	UpdatePWMOUT05_Inc									; ...dann PWM erhhen

	sts		RAM_PWMIstValue5,XH									; ...sonst SOLL-Wert in  IST-Wert laden
	ldi		XH,K_PWMSchrittweiteSchnell	
	sts		RAM_PWMSchrittweite5,XH								; Sanftanlauf auf schnell umschalten
	rjmp	UpdatePWMOUT05_End

UpdatePWMOUT05_Inc:
	lds		XL,RAM_PWMIstValue5
	lds		XH,RAM_PWMSchrittweite5
	add		XL,XH												; Schrittweite zum OCR0A addieren...
	brcs	UpdatePWMOUT05_Limit								; und prfen ob nchster Wert einen berlauf ergiebt, wenn "NEIN" dann nchste Abfrage sonst Limit erreicht...

	lds		XH,RAM_PWMSollValue5
	cp		XH,XL												; ...kein berlauf -> Abfrage ist SOLL-Wert >= IST-Wert, wenn "JA"...
	brsh	UpdatePWMOUT05_XLToPWMIstValue						; ...dann zu Ende 

UpdatePWMOUT05_Limit:
	lds		XL,RAM_PWMSollValue5								; ...NEIN bzw. berlauf SOLL-Wert in IST-Wet bernehmen

UpdatePWMOUT05_XLToPWMIstValue:
	sts		RAM_PWMIstValue5,XL

UpdatePWMOUT05_End:
	ret

; ***********************************************************************************
; * PWM Ausgabe aktualisieren fr Ausgang 6											*
; ***********************************************************************************
UpdatePWMOUT06:
	lds		XL,RAM_PWMSollValue6
	cpi		XL,K_PWMValueOff;0									; ist PWM Wert > 0...
	breq	UpdatePWMOUT06_XLToPWMIstValue						

UpdatePWMOUT06_NotZero:											; Sollwert ist nicht 0, abfrage ob PWM bereits luft
	lds		XL,RAM_PWMIstValue6
	cpi		XL,K_PWMValueOff
	brne	UpdatePWMOUT06_Run									; ...sonst weiter mit RUN dec oder inc

	lds		XL,K_PWMValueStart									; Sanftanlauf Startwert initialisieren
	ldi		XH,K_PWMSchrittweiteLangsam
	sts		RAM_PWMSchrittweite6,XH								; Sanftanlauf langsam einstellen
;	rjmp	UpdatePWMOUT06_XLToPWMIstValue
	rjmp	UpdatePWMOUT06_Inc

UpdatePWMOUT06_Run:
	lds		XL,RAM_PWMIstValue6
	lds		XH,RAM_PWMSollValue6
	cp		XL,XH												; ist IST-Wert < SOLL-Wert...
	brlo	UpdatePWMOUT06_Inc									; ...dann PWM erhhen

	sts		RAM_PWMIstValue6,XH									; ...sonst SOLL-Wert in  IST-Wert laden
	ldi		XH,K_PWMSchrittweiteSchnell	
	sts		RAM_PWMSchrittweite6,XH								; Sanftanlauf auf schnell umschalten
	rjmp	UpdatePWMOUT06_End

UpdatePWMOUT06_Inc:
	lds		XL,RAM_PWMIstValue6
	lds		XH,RAM_PWMSchrittweite6
	add		XL,XH												; Schrittweite zum OCR0A addieren...
	brcs	UpdatePWMOUT06_Limit								; und prfen ob nchster Wert einen berlauf ergiebt, wenn "NEIN" dann nchste Abfrage sonst Limit erreicht...

	lds		XH,RAM_PWMSollValue6
	cp		XH,XL												; ...kein berlauf -> Abfrage ist SOLL-Wert >= IST-Wert, wenn "JA"...
	brsh	UpdatePWMOUT06_XLToPWMIstValue						; ...dann zu Ende 

UpdatePWMOUT06_Limit:
	lds		XL,RAM_PWMSollValue6								; ...NEIN bzw. berlauf SOLL-Wert in IST-Wet bernehmen

UpdatePWMOUT06_XLToPWMIstValue:
	sts		RAM_PWMIstValue6,XL

UpdatePWMOUT06_End:
	ret

; ***********************************************************************************
; * PWM Ausgabe aktualisieren fr Ausgang 7											*
; ***********************************************************************************
UpdatePWMOUT07:
	lds		XL,RAM_PWMSollValue7
	cpi		XL,K_PWMValueOff;0									; ist PWM Wert > 0...
	breq	UpdatePWMOUT07_XLToPWMIstValue						

UpdatePWMOUT07_NotZero:											; Sollwert ist nicht 0, abfrage ob PWM bereits luft
	lds		XL,RAM_PWMIstValue7
	cpi		XL,K_PWMValueOff
	brne	UpdatePWMOUT07_Run									; ...sonst weiter mit RUN dec oder inc

	lds		XL,K_PWMValueStart									; Sanftanlauf Startwert initialisieren
	ldi		XH,K_PWMSchrittweiteLangsam
	sts		RAM_PWMSchrittweite7,XH								; Sanftanlauf langsam einstellen
;	rjmp	UpdatePWMOUT07_XLToPWMIstValue
	rjmp	UpdatePWMOUT07_Inc

UpdatePWMOUT07_Run:
	lds		XL,RAM_PWMIstValue7
	lds		XH,RAM_PWMSollValue7
	cp		XL,XH												; ist IST-Wert < SOLL-Wert...
	brlo	UpdatePWMOUT07_Inc									; ...dann PWM erhhen

	sts		RAM_PWMIstValue7,XH									; ...sonst SOLL-Wert in  IST-Wert laden
	ldi		XH,K_PWMSchrittweiteSchnell	
	sts		RAM_PWMSchrittweite7,XH								; Sanftanlauf auf schnell umschalten
	rjmp	UpdatePWMOUT07_End

UpdatePWMOUT07_Inc:
	lds		XL,RAM_PWMIstValue7
	lds		XH,RAM_PWMSchrittweite7
	add		XL,XH												; Schrittweite zum OCR0A addieren...
	brcs	UpdatePWMOUT07_Limit								; und prfen ob nchster Wert einen berlauf ergiebt, wenn "NEIN" dann nchste Abfrage sonst Limit erreicht...

	lds		XH,RAM_PWMSollValue7
	cp		XH,XL												; ...kein berlauf -> Abfrage ist SOLL-Wert >= IST-Wert, wenn "JA"...
	brsh	UpdatePWMOUT07_XLToPWMIstValue						; ...dann zu Ende 

UpdatePWMOUT07_Limit:
	lds		XL,RAM_PWMSollValue7								; ...NEIN bzw. berlauf SOLL-Wert in IST-Wet bernehmen

UpdatePWMOUT07_XLToPWMIstValue:
	sts		RAM_PWMIstValue7,XL

UpdatePWMOUT07_End:
	ret

; ***********************************************************************************
; * PWM Ausgabe aktualisieren fr Ausgang 8											*
; ***********************************************************************************
UpdatePWMOUT08:
	sbrs	R_CONTROL,F_DIPNormalMode
	rjmp	UpdatePWMOUT08_End

	lds		XL,RAM_PWMSollValue8
	cpi		XL,K_PWMValueOff;0									; ist PWM Wert > 0...
	breq	UpdatePWMOUT08_XLToPWMIstValue						

UpdatePWMOUT08_NotZero:											; Sollwert ist nicht 0, abfrage ob PWM bereits luft
	lds		XL,RAM_PWMIstValue8
	cpi		XL,K_PWMValueOff
	brne	UpdatePWMOUT08_Run									; ...sonst weiter mit RUN dec oder inc

	lds		XL,K_PWMValueStart									; Sanftanlauf Startwert initialisieren
	ldi		XH,K_PWMSchrittweiteLangsam
	sts		RAM_PWMSchrittweite8,XH								; Sanftanlauf langsam einstellen
;	rjmp	UpdatePWMOUT08_XLToPWMIstValue
	rjmp	UpdatePWMOUT08_Inc

UpdatePWMOUT08_Run:
	lds		XL,RAM_PWMIstValue8
	lds		XH,RAM_PWMSollValue8
	cp		XL,XH												; ist IST-Wert < SOLL-Wert...
	brlo	UpdatePWMOUT08_Inc									; ...dann PWM erhhen

	sts		RAM_PWMIstValue8,XH									; ...sonst SOLL-Wert in  IST-Wert laden
	ldi		XH,K_PWMSchrittweiteSchnell	
	sts		RAM_PWMSchrittweite8,XH								; Sanftanlauf auf schnell umschalten
	rjmp	UpdatePWMOUT08_End

UpdatePWMOUT08_Inc:
	lds		XL,RAM_PWMIstValue8
	lds		XH,RAM_PWMSchrittweite8
	add		XL,XH												; Schrittweite zum OCR0A addieren...
	brcs	UpdatePWMOUT08_Limit								; und prfen ob nchster Wert einen berlauf ergiebt, wenn "NEIN" dann nchste Abfrage sonst Limit erreicht...

	lds		XH,RAM_PWMSollValue8
	cp		XH,XL												; ...kein berlauf -> Abfrage ist SOLL-Wert >= IST-Wert, wenn "JA"...
	brsh	UpdatePWMOUT08_XLToPWMIstValue						; ...dann zu Ende 

UpdatePWMOUT08_Limit:
	lds		XL,RAM_PWMSollValue8								; ...NEIN bzw. berlauf SOLL-Wert in IST-Wet bernehmen

UpdatePWMOUT08_XLToPWMIstValue:
	sts		RAM_PWMIstValue8,XL

UpdatePWMOUT08_End:
	ret

/*
; ***********************************************************************************
; * Multifunktions Key Management													*
; ***********************************************************************************
MFKeyManagement:
	lds		XL,RAM_BufferFunkCodeTaste
	cpi		XL,K_SWMF
	breq	MFKeyManagement_CheckSource
	rjmp	MFKeyManagement_End

MFKeyManagement_CheckSource:
	sbrs	R_CONTROL,F_TasteFunk						; abfrage ob Taster auf Handsender gedrckt wurde
	rjmp	MFKeyManagement_Go

MFKeyManagement_KeyByFunk:
	sbrs	R_CONTROL,F_TasteKurz						; Bei Taster auf Handsender auf kurzen Tastendruck reagieren
	rjmp	MFKeyManagement_End

	cbr		R_CONTROL,(1<<F_TasteKurz)
	sbr		R_CONTROL,(1<<F_BluetoothMode)
	rjmp	MFKeyManagement_Analysis

MFKeyManagement_Go:
	sbrs	R_CONTROL,F_TasteLang						; Bei Taster auf Bedieneinheit auf langen Tatsendruck reagieren
	rjmp	MFKeyManagement_End

	cbr		R_CONTROL,(1<<F_TasteLang)
	ldi		XL,Low(K_TimeFactorySettings)													; 
	ldi		XH,High(K_TimeFactorySettings)
	sts		RAM_TimeCounterL,XL
	sts		RAM_TimeCounterH,XH	
	cbr		R_STATUS,(1<<F_TimeOut)						; und Flag setzen
	cbr		R_CONTROL,(1<<F_BluetoothMode)
	cbr		R_CONTROL,(1<<F_LearnInMode)
	cbr		R_CONTROL,(1<<F_DeleteMemoryMode)
	cbr		R_CONTROL,(1<<F_FactoryResetMode)
	rcall	LCD_Clear
	rcall	LCD_Home
	rcall	LCD_ShowServiceMode
	rcall	LCD_InitDoubleHighSet1

MFKeyManagement_Loop:
MFKeyManagement_CheckBluetoothLoop:
	sbic	SWMFPin,SWMF
	rjmp	MFKeyManagement_Analysis

	wdr
	lds		XL,RAM_TimeCounterL
	lds		XH,RAM_TimeCounterH

	sbrc	R_CONTROL,F_BluetoothMode
	rjmp	MFKeyManagement_CheckLearnInLoop	

	sbrc	R_CONTROL,F_LearnInMode
	rjmp	MFKeyManagement_CheckDeleteMemoryLoop

	sbrc	R_CONTROL,F_DeleteMemoryMode
	rjmp	MFKeyManagement_CheckRestoreModeLoop


	; ***********************************
	; * Warten auf Bluetooth Mode		*
	; ***********************************
	ldi		YH,High(K_TimeFactorySettings-K_TimeBluetoothMode)
	cpi		XL,Low(K_TimeFactorySettings-K_TimeBluetoothMode)
	cpc		XH,YH
	brsh	MFKeyManagement_Loop

	; ***********************************
	; * Bluetooth Mode					*
	; ***********************************	
	sbr		R_CONTROL,(1<<F_BluetoothMode)
	rcall	LCD_ShowBluetoothMode						; Anzeige auf LCD

	sbrc	R_CONTROL,F_BluetoothMode
	rjmp	MFKeyManagement_CheckRestoreModeLoop	

MFKeyManagement_CheckLearnInLoop:
	; ***********************************
	; * Warten auf LernIn Mode			*
	; ***********************************
	ldi		YH,High(K_TimeFactorySettings-K_TimeLearnInMode)
	cpi		XL,Low(K_TimeFactorySettings-K_TimeLearnInMode)
	cpc		XH,YH
	brsh	MFKeyManagement_Loop

	; ***********************************
	; * LernIn Mode						*
	; ***********************************	
	cbr		R_CONTROL,(1<<F_BluetoothMode)
	sbr		R_CONTROL,(1<<F_LearnInMode)
	rcall	LCD_ShowLearnInMode							; Anzeige auf LCD

MFKeyManagement_CheckDeleteMemoryLoop:
	; ***********************************
	; * Warten auf Delete Memory Mode	*
	; ***********************************
	ldi		YH,High(K_TimeFactorySettings-K_TimeDeleteMemoryMode)
	cpi		XL,Low(K_TimeFactorySettings-K_TimeDeleteMemoryMode)
	cpc		XH,YH
	brsh	MFKeyManagement_Loop

	; ***********************************
	; * Delete Memory Mode				*
	; ***********************************
	cbr		R_CONTROL,(1<<F_LearnInMode)
	sbr		R_CONTROL,(1<<F_DeleteMemoryMode)
	rcall	LCD_ShowDeleteMemoryMode							; Anzeige auf LCD

	; *******************************************
	; * Warten auf Restore Factory Settings		*
	; *******************************************
MFKeyManagement_CheckRestoreModeLoop:
	sbrs	R_STATUS,F_TimeOut
	rjmp	MFKeyManagement_Loop

	; *******************************************
	; * Restore Factory Settings				*
	; *******************************************
	cbr		R_CONTROL,(1<<F_DeleteMemoryMode)
	sbr		R_CONTROL,(1<<F_FactoryResetMode)
	rcall	LCD_FactoryReset									; Anzeige auf LCD

MFKeyManagement_WaitKeyNot:										; Werten bis Taster wieder losgelassen wird
	wdr
	sbis	SWMFPin,SWMF	
	rjmp	MFKeyManagement_WaitKeyNot

	; ***********************************
	; * Auswertung						*
	; ***********************************	
MFKeyManagement_Analysis:
	ldi		XL,0
	sts		RAM_BufferFunkCodeTaste,XL

	sbrs	R_CONTROL,F_BluetoothMode							; Wurde LearnIn Mode aktiviert nichts machen
	rjmp	MFKeyManagement_AnalysisNotBluetooth

	cbr		R_CONTROL,(1<<F_BluetoothMode)
	rcall	BluetoothEnable										; Flag wird in andetrer Rotine ausgewertet
	rjmp	MFKeyManagement_Ready

MFKeyManagement_AnalysisNotBluetooth:
	sbrs	R_CONTROL,F_LearnInMode								; Wurde LearnIn Mode aktiviert nichts machen
	rjmp	MFKeyManagement_Analysis_NotLearnIn

	cbr		R_CONTROL,(1<<F_LearnInMode)
	rcall	LearnInDevice										; Flag wird in andetrer Rotine ausgewertet
	rjmp	MFKeyManagement_Ready
		
MFKeyManagement_Analysis_NotLearnIn:
	sbrs	R_CONTROL,F_DeleteMemoryMode						; Wurde Factory Reset Mode aktiviert Werkseinstellungen laden
	rjmp	MFKeyManagement_Analysis_NotDeleteMemory

	cbr		R_CONTROL,(1<<F_DeleteMemoryMode)
	rcall	DeleteMemory
	rjmp	MFKeyManagement_Ready

MFKeyManagement_Analysis_NotDeleteMemory:
	sbrs	R_CONTROL,F_FactoryResetMode						; Wurde Factory Reset Mode aktiviert Werkseinstellungen laden
	rjmp	MFKeyManagement_Ready

	cbr		R_CONTROL,(1<<F_FactoryResetMode)
	rcall	RestoreFactorySettings
	rjmp	MFKeyManagement_Ready

MFKeyManagement_Ready:
	rcall	LCD_InitDoubleHighClear

MFKeyManagement_End:
	ret
*/
